class DeployInfo(dict):
    def __init__(self, deployment_paths):

        deploypaths = []
        for deploy_path in deployment_paths:
            path = dict()
            path['name'] = deploy_path.Name
            path['is_default'] = deploy_path.IsDefault
            path['service_name'] = deploy_path.DeploymentService.Model
            path['attributes'] = dict()
            for attribute in deploy_path.DeploymentService.Attributes:
                path['attributes'][attribute.Name] = attribute.Value
            deploypaths.append(path)
        dict.__init__(self, deploypaths=deploypaths)



import json

from cloudshell.api.cloudshell_api import SandboxDataKeyValue
from cloudshell.helpers.save_workflow.deploy_info import DeployInfo


def save_app_deployment_info(sandbox, components=None):
    for app in sandbox.automation_api.GetReservationDetails(sandbox.id).ReservationDescription.Apps:
        serialized_deployment_info = json.dumps(DeployInfo(app.DeploymentPaths))

        key_value = SandboxDataKeyValue(app.Name, serialized_deployment_info)

        sandbox.automation_api.SetSandboxData(sandbox.id, [key_value])



import os
import json
import requests

from cloudshell.helpers.app_import.build_app_xml import app_template
from cloudshell.helpers.app_import.upload_app_xml import upload_app_to_cloudshell
from cloudshell.api.cloudshell_api import InputNameValue


class SaveAppUtility:
    def __init__(self, sandbox, resource_name, server_address, admin_user, admin_password, display_image_url='', new_app_name='', save_as=False):
        self.sandbox = sandbox
        self.resource_name = resource_name
        self.app_name = ''
        self.AppTemplateName = ''
        self.new_app_name = ''

        self.api_missing = False

        for vm in self.sandbox.automation_api.GetReservationDetails(self.sandbox.id).ReservationDescription.Resources:
            if vm.Name == self.resource_name:
                self.app_name = vm.AppDetails.AppName
                try:
                    self.AppTemplateName = vm.AppTemplateName
                    if new_app_name == '':
                        if save_as:
                            if self.app_name == self.AppTemplateName:
                                self.api_missing = True
                            else:
                                self.new_app_name = self.app_name
                        else:
                            self.new_app_name = self.AppTemplateName
                    else:
                        self.new_app_name = new_app_name
                except Exception as e:
                    self.new_app_name = self.app_name
                if self.api_missing:
                    raise Exception("Stopping Save As because App's name was not changed prior to deployment.\n"
                                    "Use the 'NewAppName' custom attribute on the command to override.")

        self.server_address = server_address
        self.admin_user = admin_user
        self.admin_password = admin_password
        self.display_image_name = 'vm.png'
        self.display_image_url = display_image_url

        self.display_image_result = None
        self.deploy_info = None
        self.saved_app_info = None
        self.app_xml = None

    def verify_deploy_info_and_display_image(self):
        self.get_deployment_info()
        if self.deploy_info is None:
            raise Exception("Could not locate Sandbox information on {}, App must be deployed by Setup script to use this functionality.\n".format(self.resource_name))

        self.get_display_image()

    def get_deployment_info(self):
        for keyValue in self.sandbox.automation_api.GetSandboxData(self.sandbox.id).SandboxDataKeyValues:
            if keyValue.Key == self.app_name:
                self.deploy_info = json.loads(keyValue.Value)
                break

    def get_display_image(self):
        self.display_image_result = self.sandbox.automation_api.GetReservationAppImage(self.sandbox.id,
                                                                                       self.resource_name).AppTemplateImage

        if self.display_image_result == '':
            try:
                if self.display_image_url != '':
                    self.display_image_result = requests.get(self.display_image_url, allow_redirects=True).content
                    self.display_image_name = os.path.basename(self.display_image_url)
                else:
                    self.display_image_result = None
                    self.display_image_name = 'vm.png'
            except:
                self.display_image_result = None
                self.display_image_name = 'vm.png'

    def save_app_info(self, delete):
        command = [x.Name for x in self.sandbox.automation_api.GetResourceConnectedCommands(self.resource_name).Commands
                   if x.Name == 'create_app_image']

        if len(command) == 1:
            if delete:
                inputs = [InputNameValue(Name='delete_old_image', Value='True')]
            else:
                inputs = [InputNameValue(Name='delete_old_image', Value='False')]
            self.saved_app_info = json.loads(self.sandbox.automation_api.ExecuteResourceConnectedCommand(self.sandbox.id,
                                                                                                         self.resource_name,
                                                                                                         'create_app_image',
                                                                                                         'connectivity',
                                                                                                         inputs).Output)
        else:
            raise Exception("Operation not supported by Cloud Provider\n")

    def create_app_xml(self):
        resource = self.sandbox.automation_api.GetResourceDetails(self.resource_name)

        app_attributes = dict()
        for attr in resource.ResourceAttributes:
            app_attributes[attr.Name] = attr.Value

        app_categories = ['Applications']

        if self.saved_app_info is not None:
            for deploy_path in self.deploy_info['deploypaths']:
                if deploy_path['is_default']:
                    for key, value in self.saved_app_info.iteritems():
                        # patch for AWS
                        if 'AWS' in key:
                            deploy_path['attributes']['AWS AMI Id'] = value
                        else:
                            deploy_path['attributes'][key] = value

        self.app_xml = app_template(self.new_app_name, self.deploy_info['deploypaths'], app_categories, app_attributes,
                                    resource.ResourceModelName, resource.DriverName,
                                    resource.VmDetails.CloudProviderFullName, self.display_image_name)

    def upload_app(self):
        result = upload_app_to_cloudshell(self.sandbox.automation_api, self.sandbox.id, self.new_app_name, self.app_xml,
                                          self.server_address, self.admin_user, self.admin_password, self.display_image_result, self.display_image_name)
        if result is None:
            self.sandbox.automation_api.WriteMessageToReservationOutput(self.sandbox.id,
                                                                        "App '{}' has been updated from instance '{}'\n".format(self.new_app_name, self.resource_name))
        else:
            raise Exception("Error uploading App to CloudShell\n{}".format(result))

    def save_flow(self, delete=False):
        if not self.api_missing:
            self.verify_deploy_info_and_display_image()
            self.save_app_info(delete)
            self.create_app_xml()
            self.upload_app()
            if delete:
                self.sandbox.automation_api.RefreshAppInBlueprints(self.AppTemplateName)

    def save_flow_just_app(self, update=False):
        if not self.api_missing:
            self.verify_deploy_info_and_display_image()
            self.create_app_xml()
            self.upload_app()
            if update:
                self.sandbox.automation_api.RefreshAppInBlueprints(self.AppTemplateName)



import os
import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

with open(os.path.join('version.txt')) as version_file:
    version_from_file = version_file.read().strip()

with open('requirements.txt') as f_required:
    required = f_required.read().splitlines()

with open('test_requirements.txt') as f_tests:
    required_for_tests = f_tests.read().splitlines()

setuptools.setup(
    name="cloudshell-app-helper",
    version=version_from_file,
    author="Quali",
    author_email="support@qualisystems.com",
    description="CloudShell package to assist with creating Apps.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    package_data={'': ['*.txt']},
    include_package_data=True,
    tests_require=required_for_tests,
    install_requires=required,
    classifiers=[
        "Programming Language :: Python :: 2.7",
        "License :: OSI Approved :: Apache Software License",
    ],
    python_requires='>=2.7.0,<2.8.0',
)


import tempfile
import os
import zipfile
import requests
import shutil

from images import vm_image


def upload_app_to_cloudshell(cs_api, reservation_id, app_name, app_xml_content, server, user="admin", password="admin", display_image_result=None, display_image_name='vm.png'):
    """
    :param CloudShellAPISession cs_api:
    :param string reservation_id:
    :param string app_name:
    :param string app_xml_content:
    :param string server:
    :param string user:
    :param string password:
    :param image display_image_result
    :param string display_image_name
    :return:
    """
    metadata = """<?xml version="1.0" encoding="utf-8"?>
<Metadata xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns="http://schemas.qualisystems.com/PackageMetadataSchema.xsd">
<CreationDate>9/13/2019 4:45:59 PM</CreationDate>
<ServerVersion>9.2.0</ServerVersion>
<PackageType>CloudShellPackage</PackageType>
</Metadata>
    """

    # script_working_dir = os.path.dirname(os.path.abspath(__file__))
    working_dir = tempfile.mkdtemp()
    app_template_file = "{}/{}".format(working_dir, app_name)
    blueprint_zip_file = "{}/Blueprint.zip".format(working_dir)

    display_image_file = os.path.join(working_dir, display_image_name)

    metadata_file = "{}/metadata.xml".format(working_dir)
    with open(app_template_file, "w") as app_xml:
        app_xml.write(app_xml_content)

    if os.path.exists(blueprint_zip_file):
        os.remove(blueprint_zip_file)

    if os.path.exists(display_image_file):
        os.remove(display_image_file)

    fh = open(display_image_file, "wb")
    if display_image_result is not None:
        fh.write(display_image_result.decode('base64'))
    else:
        fh.write(vm_image.decode('base64'))
    fh.close()

    zip_file = zipfile.ZipFile(blueprint_zip_file, "w")
    zip_file.write(app_template_file, arcname="App Templates\\{}.xml".format(app_name))

    zip_file.write(display_image_file, arcname="App Templates\\{}".format(display_image_name))

    open(metadata_file, "w").write(metadata)
    zip_file.write(metadata_file, arcname="metadata.xml")
    zip_file.close()
    zip_content = open(blueprint_zip_file, "rb").read()
    shutil.rmtree(working_dir)
    authentication_code = requests.put("http://{}:9000/Api/Auth/Login".format(server),
                                       {"username": user, "password": password, "domain": "Global"}).content

    result = requests.post("http://{}:9000/API/Package/ImportPackage".format(server),
                           headers={"Authorization": "Basic {}".format(authentication_code[1:-1])},
                           files={"QualiPackage": zip_content})
    if 'false' in result.content:
        raise Exception('Issue importing App XML into Cloudshell: ' + result.content)
    if result.status_code >= 300:
        return result.content
    else:
        return None


def attribute(name, value, overwrite=False):
    xml ="""
    <Attribute Name="{Name}" Value="{Value}" {overwrite}/>
    """
    return xml.format(Name=name, Value=value, overwrite='Override="true"' if overwrite else "")


def deployment_path(deploy_path, cloud_provider):
    xml = """
        <DeploymentPath Name="{dn}" Default="false">
        <DeploymentService Name="{dsn}" CloudProvider="{cp}">
        <Attributes>
        {DeployAttributes}
        </Attributes>
        </DeploymentService>
        </DeploymentPath>
    """
    return xml.format(DeployAttributes="\n".join([attribute(x, y, True) for x, y in deploy_path['attributes'].iteritems()]),
                      cp=cloud_provider, dn=deploy_path['name'], dsn=deploy_path['service_name'])


def deployment_paths(deploy_paths, cloud_provider):
    xml = """
    {DeployPaths}
"""
    return xml.format(DeployPaths="\n".join([deployment_path(x, cloud_provider) for x in deploy_paths]))


def app_resource(attributes, model, driver):
    xml = """
        <AppResources>
      <AppResource ModelName="{model}" Driver="{driver}">
        <Attributes>
          {Attributes}
        </Attributes>
      </AppResource>
    </AppResources>
    
    """
    return xml.format(Attributes="\n".join(attribute(x, y, True) for x, y in attributes.iteritems()), model=model, driver=driver)


def app_resource_info(app_name, deploy_paths, app_attributes, model, driver, cloud_provider, image_name):
    xml = """
  <AppResourceInfo Name="{AppName}" ImagePath="{img}">
    {AppResource}
    <DeploymentPaths>
    {DeploymentPath}
    
    </DeploymentPaths>
    </AppResourceInfo>
    """
    return xml.format(AppName=app_name, img=image_name, AppResource=app_resource(app_attributes, model, driver), DeploymentPath=deployment_paths(deploy_paths, cloud_provider))


def add_category(category):
    xml = """
    <Category>{Category}</Category>
    """
    return xml.format(Category=category)


def categories_info(categories):
    xml = """
      <Categories>
        {Categories}
  </Categories>
    
    """
    return xml.format(Categories="\n".join([add_category(x) for x in categories]))


def app_template(app_name, deploy_paths, categories, app_attributes, model, driver, cloud_provider, image_name):
    """

    :return:
    """
    xml ="""<?xml version="1.0" encoding="utf-8"?>
<AppTemplateInfo xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
    {AppResourceInfo}

        {Categories}
    </AppTemplateInfo>
    """
    app_resource = app_resource_info(app_name, deploy_paths, app_attributes, model, driver, cloud_provider, image_name)
    categories = categories_info(categories)

    return xml.format(AppResourceInfo=app_resource, Categories=categories if categories else "")

